/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.services.client.rpc;

import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.rpc.IsSerializable;
import java.io.Serializable;
import java.util.ArrayList;

/**
 * For internal use only by implementing SOAFaces container. Not for use by application
 * developers.
 * 
 * @author Sam Taha
 */
final public class GenericDataUtil {
  
  private GenericDataUtil() {}

  public static Object getPayload(GenericDataWrapper proxy)
  {
    if(proxy == null)
      return null;

    Object result = null;
    
    if(proxy._Pojo != null)
      result = proxy._Pojo;
    else if(proxy._PojoAr != null)
      result = proxy._PojoAr;
    else if(proxy._PojoSerial != null)
      result = proxy._PojoSerial;
    else if(proxy._PojoSerialAr != null)
      result = proxy._PojoSerialAr;
    else if(proxy._PojoSerialArAr != null)
      result = proxy._PojoSerialArAr;
    
    return result;
 }
  
  public static GenericDataWrapper createGenericDataWrapper(Object obj) 
  {
    if (obj == null)
      return null;
 
    GenericDataWrapper proxy = new GenericDataWrapper();
    
    if (obj instanceof IsSerializable) {
      proxy._Pojo = (IsSerializable) obj;
      //Window.alert("here 1");
    } else if (obj instanceof IsSerializable[]) {
      proxy._PojoAr = (IsSerializable[]) obj;
      //Window.alert("here 2");
    } else if(obj.getClass().getName().equals("[Ljava.lang.Object;")) {
      //Very important this else if is before Serializble condition below
      //because arrays in java always implement Serializable.
      //Window.alert("here 6");
      uncoilObjectArray(obj, proxy);
    } else if (obj instanceof JSONObject) {
      proxy._Json = ((JSONObject) obj).toString();
    } else if (obj instanceof Serializable[][]) {
      proxy._PojoSerialArAr = (Serializable[][]) obj;
      //Window.alert("here 5");
    } else if (obj instanceof Serializable[]) {
      proxy._PojoSerialAr = (Serializable[]) obj;
      //Window.alert("here 4");
    } else if(obj instanceof Serializable) {
      proxy._PojoSerial = (Serializable) obj;
      //Window.alert("here 3");
    } else {
      //return null; //means the argument is null itself
      //Window.alert("SOAFaces Error: Not able to serialize this data - " + obj.getClass().getName());
      throw new UniversalClientRemoteException("SOAFaces Error: Not able to serialize object - " + obj.getClass().getName());
    }

    return proxy;
  }

  private static void uncoilObjectArray(Object obj, GenericDataWrapper proxy) throws UniversalClientRemoteException
  {
    //Must be a messaging related payload. So we simulate passing
    //an Object[] by putting it into a List. This is needed to allow
    //for proper GWT serialization.

    Object[] myObjAr = (Object[]) obj;
    //put the Object array elements into a List

    int arLength = myObjAr.length;
    if(arLength > 0)
    {
      proxy._jsonAr = new ArrayList(arLength);
      proxy._isserAr = new ArrayList(arLength);
      proxy._isserArAr = new ArrayList(arLength);
      proxy._serAr = new ArrayList(arLength);
      proxy._serArAr = new ArrayList(arLength);
      proxy._serArArAr = new ArrayList(arLength);
    }

    Object objVal = null;
    for(int ii=0; ii<arLength; ii++)
    {
      objVal = myObjAr[ii];

      //This lets us mix JSON and POJOs in a multi arguement method
      if(objVal == null)
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add(null);
        proxy._isserArAr.add(null);
        proxy._serAr.add(null);
        proxy._serArAr.add(null);
        proxy._serArArAr.add(null);
      }
      else if(objVal instanceof JSONObject)
      {
        JSONObject jsonObj = (JSONObject) objVal;
        objVal = new JSONTypeWrapper(jsonObj.toString());
        proxy._jsonAr.add((JSONTypeWrapper) objVal);
        proxy._isserAr.add(null);
        proxy._isserArAr.add(null);
        proxy._serAr.add(null);
        proxy._serArAr.add(null);
        proxy._serArArAr.add(null);

      }
      else if(objVal instanceof IsSerializable)
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add((IsSerializable) objVal);
        proxy._isserArAr.add(null);
        proxy._serAr.add(null);
        proxy._serArAr.add(null);
        proxy._serArArAr.add(null);
      }
      else if(objVal instanceof IsSerializable[])
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add(null);
        proxy._isserArAr.add((IsSerializable[]) objVal);
        proxy._serAr.add(null);
        proxy._serArAr.add(null);
        proxy._serArArAr.add(null);
      }
      else if(objVal instanceof Serializable[][])
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add(null);
        proxy._isserArAr.add(null);
        proxy._serAr.add(null);
        proxy._serArAr.add(null);
        proxy._serArArAr.add((Serializable[][]) objVal);
      }
      else if(objVal instanceof Serializable[])
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add(null);
        proxy._isserArAr.add(null);
        proxy._serAr.add(null);
        proxy._serArAr.add((Serializable[]) objVal);
        proxy._serArArAr.add(null);
      }
      else if(objVal instanceof Serializable)
      {
        proxy._jsonAr.add(null);
        proxy._isserAr.add(null);
        proxy._isserArAr.add(null);
        proxy._serAr.add((Serializable) objVal);
        proxy._serArAr.add(null);
        proxy._serArArAr.add(null);
      }
      else {
        throw new UniversalClientRemoteException("SOAFaces Error: Not able to serialize object in Object[] - " + objVal.getClass().getName());
      }
    }

    proxy._pseudoObjectAr = true; //simple flag that _List is used to hold Object[] elements
  }
}
